package zenus;

=copyright
	Copyright 2008 Tom Chiverton 
	 
	This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
=cut

require Exporter;
use SOAP::DateTime;
use SOAP::Lite;	
use Data::Dumper;
use Time::localtime;

@ISA=qw(Exporter);

my $version="2.1.5";
my $client="Zenus";
my $clientIsDebug="false";

my $hideDebug=1;

# https://webservices.zen.co.uk/broadband/v3.00/serviceengine.asmx
# https://webservices.zen.co.uk/broadband/v3.00/serviceengine.asmx?WSDL

my $WSA_NS = 'http://schemas.xmlsoap.org/ws/2004/08/addressing';
my $WS_NS =  'https://webservices.zen.co.uk/broadband/v3.00/';
my $BASE_URL='https://webservices.zen.co.uk/Broadband/v3.00/serviceengine.asmx?op=';

my $ACCESS_URL = $BASE_URL.'Authenticate';
my $CLIENT_URL = $BASE_URL.'ValidateClient';
my $USAGE_URL =  $BASE_URL.'GetUsage';
my $GETDEFAULTBROADBANDACCOUNT_URL =$BASE_URL.'GetDefaultBroadbandAccount';
my $SETDEFAULTBROADBANDACCOUNT_URL =$BASE_URL.'GetAuthorisedBroadbandAccounts';
my $GETBROADBANDACCOUNTS_URL	   =$BASE_URL.'SetDefaultBroadbandAccount';  

my @account_formatted;
my $username;
my $password;
my $clientId;
my $authToken;

sub setDebug{
	$hideDebug=0;
	$clientIsDebug="true";
}

sub estimateRemaining{
	my $used=shift;
	my $avail=shift;
	
	if ($used eq '' or $avail eq ''){
		die 'No used or available value';
	}
	
	$day = localtime->mday();
	$fday=($day-1)*24 + localtime->hour() + ( localtime->min()/60 ) ;
	$month = localtime->mon();
	$year = localtime->year();
	if (($month == 4) || ($month == 6) || ($month == 9) || ($month == 11)) {
		$length = 30;
	}elsif($month == 2 && _isLeapYear($year)) {
		$length = 29;
	}elsif ($month == 2) {
		$length = 28;
	} else {
		$length = 31;
	}
	$flength=$length*24;
	$percentmonthused = $fday/$flength*100;
	$percentmonthleft = 100-$percentmonthused;
	
	$percentused=0;
	if ($avail > 0) { $percentused = $used/$avail*100; }
	$percentleft = 100-$percentused;
	
	$dailyallowance = $avail/$length;
	$dailyusagetodate = $used/$day;
	$hourlyusagetodate = $used/$fday;
	if (!( ($length-$day) eq 0)){
		$dailyusageremaining = ($avail-$used)/($length-$day);
	}else{
		$dailyusageremaining = $avail-$used
	}
	
	$estEndOfMonthUsage=$hourlyusagetodate*$flength;
	
	if ($dailyusagetodate eq 0){
		$exhaustdate=-0.00000000001;
	}else{
		$exhaustdate=($avail-$used)/$dailyusagetodate;
	}

	my @result;
	push @result,$day;
	push @result,$percentmonthused;
	push @result,$dailyusagetodate;
	push @result,$dailyusageremaining;
	push @result,$exhaustdate;
	push @result,$estEndOfMonthUsage;
	push @result,$length;
	return @result;	
}

sub _isLeapYear
{
   my $year = shift;
   return 0 if $year % 4;
   return 1 if $year % 100;
   return 0 if $year % 400;
   return 1;
}


sub login{
	$username=shift;
	$password=shift;

	if ($username eq '' || $password eq ''){
		die 'No username or password';
	}

	print "logging in with $username,$password\n" unless $hideDebug;
	
	my $result = SOAP::Lite
      	->proxy($ACCESS_URL)
        ->uri($WS_NS)
	->on_action(sub{join '', @_})	
	->Authenticate(
		SOAP::Data->prefix('e')
			->uri($WS_NS)->type('')->name("username" => $username),
		SOAP::Data->prefix('e')
			->uri($WS_NS)->type('')->name("password" => $password),
		SOAP::Header->uri($WSA_NS)->prefix("wsa")->name("Action" => $WS_NS),
		SOAP::Header->uri($WSA_NS)->prefix("wsa")->name("MessageID" => "urn:uuid:$$\@perl-client"),
		SOAP::Header->attr({"xmlns" => $WSA_NS})
			->name("ReplyTo" => [ SOAP::Header->name("Address" =>  'http://schemas.xmlsoap.org/ws/2004/08/addressing/role/anonymous') ]),
		SOAP::Header->uri($WSA_NS)->prefix("wsa")
			->name("To" => $ACCESS_URL),
		SOAP::Header->type('')
			->attr({"xmlns" => "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"})
			->name("Security" => [
				SOAP::Header->type('')->name("UsernameToken" => [
						SOAP::Header->type('')->name("Username" => $username),
						SOAP::Header->type('')->attr(
							{ "Type" => 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText'}
						)->name("Password" => $password)
					]
				)
			]),	
	);
	 
	if ($result->fault){
		_dieWithError($result);		
	}
	
	$authToken=$result->dataof('//AuthenticateResponse/AuthenticateResult')->value;
	
	print "Login result :".$authToken."\n" unless $hideDebug;

	$result = SOAP::Lite
      	->proxy($CLIENT_URL)
        ->uri($WS_NS)
	->on_action(sub{join '', @_})	
	->ValidateClient(
		#Create call data 
		SOAP::Data->prefix('e')->uri($WS_NS)
			->name("ClientName" => "$client"),
		SOAP::Data->prefix('e')->uri($WS_NS)
			->name("ClientVersion" => "$version"),
		SOAP::Data->prefix('e')->uri($WS_NS)
			->name("ClientIsBeta" => $clientIsDebug),
		SOAP::Data->prefix('e')->uri($WS_NS)
			->name("AuthenticationGUID" =>$authToken), 
		#Commence pointless headers
		SOAP::Header->uri($WSA_NS)->prefix("wsa")->name("Action" => $WS_NS),
		SOAP::Header->uri($WSA_NS)->prefix("wsa")->name("MessageID" => "urn:uuid:$$\@perl-client"),
		SOAP::Header->attr({"xmlns" => $WSA_NS})
			->name("ReplyTo" => [ SOAP::Header->name("Address" =>  'http://schemas.xmlsoap.org/ws/2004/08/addressing/role/anonymous') ]),
		SOAP::Header->uri($WSA_NS)->prefix("wsa")
			->name("To" => $CLIENT_URL),
		SOAP::Header->type('')
			->attr({"xmlns" => "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"})
			->name("Security" => [
				SOAP::Header->type('')->name("UsernameToken" => [
						SOAP::Header->type('')->name("Username" => $username),
						SOAP::Header->type('')->attr(
							{ "Type" => 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText'}
						)->name("Password" => $password)
					]
				)
			]),
	);
		 
	if ($result->fault){
		_dieWithError($result);		
	}
	
	$clientId=$result->dataof('//ValidateClientResponse/ValidateClientResult')->value;
	
	print "Client ID :".$authToken."\n" unless $hideDebug;
}


sub getUsage{
	my $account=shift;
	
	if ($account eq ''){
		print "using default account, " unless $hideDebug;
		($account)=getDefaultAccount();
		if ($account eq ''){
			print "setting default account, " unless $hideDebug;
			($account)=setDefaultAccount();
			print "set to $account\n" unless $hideDebug;
		}	
	}
	
	push @account_formatted, SOAP::Data->prefix('e')->uri($WS_NS)->name("string" => $account);
	
	print "account $account\n"."\n" unless $hideDebug;
	
	my $result = SOAP::Lite
      	->proxy($USAGE_URL)
        ->uri($WS_NS)
	->on_action(sub{join '', @_})	
	->GetUsage(
		#Create call data 
		SOAP::Data->prefix('e')->uri($WS_NS)->type('')
			->name("adslAccounts")->value(\@account_formatted),

		SOAP::Data->prefix('e')->uri($WS_NS)->type("xsd:dateTime")
			->name("dateTimeFrom" => ConvertDate('1st')),

		SOAP::Data->prefix('e')->uri($WS_NS)->type("xsd:dateTime")
			->name("dateTimeTo" => scalar ConvertDate('1st')),
		_SOAPAuthArgs($USAGE_URL)
	);
	
	unless ($result->fault) {
		my $r=$result;
		print Dumper(\$r->dataof('//GetUsageResponse/')) unless $hideDebug;
	
		my $statuscode = $r->dataof('//GetUsageResponse/GetUsageResult/Statistic/StatusCode')->value;
	
		if($statuscode ne 'Ok')
		{
			my $stmessage = $r->valueof('//StatusMessages/string');
			die "Remote error: [$statuscode] $stmessage\n";
		}	
			
		if (! $r->dataof('//GetUsageResponse/GetUsageResult/Statistic/AdditionalDetails/AllowanceDetails') ){
			die "Invalid account $account";
		}	
				
		my $available = $r->dataof('//GetUsageResponse/GetUsageResult/Statistic/AdditionalDetails/AllowanceDetails')->attr->{UsageAllowance};
		my $used = $r->dataof('//GetUsageResponse/GetUsageResult/Statistic/UsageStatistics/DownloadUsage')->value;
		my $username = $r->dataof('//GetUsageResponse/GetUsageResult/Statistic/')->attr->{BroadbandAccount};
		my $uploadAmount = $r->dataof('//GetUsageResponse/GetUsageResult/Statistic/UsageStatistics/UploadUsage')->value;
		
		my $percent_used = 0;
		if ($available > 0) {
			$percent_used = ($used * 100 / $available);
			print "$username\t$percent_used %\t$used/$available mb up:$uploadAmount\n" unless $hideDebug;
		}else{
			print "$username\t (no available amount) up:$uploadAmount\n" unless $hideDebug;
		}
		$used=$used/1024;
		$available=$available/1024;
		$uploadAmount = $uploadAmount/1024;
		
		my @result;
		push @result,$username;
		push @result,$used;
		push @result,$available;
		push @result,$percent_used;
		push @result,$uploadAmount;
			
		return @result;							
	}else{
		_dieWithError($result);	
	}
	
}

sub setDefaultAccount{
	my $accnt_id=shift;
	if ($accnt eq ''){
		my (@accnts)=getAccounts();
		$accnt_id=$accnts[0];
		print "setDefaultAccount defaulted to $accnt_id\n" unless $hideDebug;
	}
	
	my $result = SOAP::Lite
      	->proxy($SETDEFAULTBROADBANDACCOUNT_URL)
        ->uri($WS_NS)
		->on_action(sub{join '', @_})	
		->SetDefaultBroadbandAccount(
		#Create call data 		
		SOAP::Data->prefix('e')->uri($WS_NS)->type('')
			->name("DSLUsername")->value($accnt_id),
		_SOAPAuthArgs($SETDEFAULTBROADBANDACCOUNT_URL)
	);
	_dieWithError($result) if $result->fault;
	return $accnt_id;
}

sub getAccounts{
	my $result = SOAP::Lite
      	->proxy($GETBROADBANDACCOUNTS_URL)
        ->uri($WS_NS)
        #gave up parsing SOAP:SOM responce, we'll use XML instead
        ->outputxml('true')
		->on_action(sub{join '', @_})	
		->GetAuthorisedBroadbandAccounts(
		_SOAPAuthArgs($GETBROADBANDACCOUNTS_URL)	
	);
	
	die "Error listing accounts" unless $result =~m/DSLUsername/;
	
	$result=~m!DSLUsername>([^/]*)</DS!g;
		
	print "accounts $1 $2 $3 ... \n" unless $hideDebug;	
		
	my $result;
	push @result,$1;
	push @result,$2;
	push @result,$3;
	push @result,$4;
	push @result,$5;
	push @result,$6;
	push @result,$7;
	push @result,$8;
	push @result,$9;	
	return @result;
}

sub getDefaultAccount{
	my $result = SOAP::Lite
      	->proxy($GETDEFAULTBROADBANDACCOUNT_URL)
        ->uri($WS_NS)
	->on_action(sub{join '', @_})	
	->GetDefaultBroadbandAccount(
		_SOAPAuthArgs($GETDEFAULTBROADBANDACCOUNT_URL)
	);
	
	unless ($result->fault) {
		my $accnt = $result->dataof('//GetDefaultBroadbandAccountResponse/GetDefaultBroadbandAccountResult');
		if ($accnt != ''){
			$accnt=$accnt->value;
		}else{
			$accnt ='';
		}
		my @result;
		push @result,$accnt;			
		return @result;
	}else{
		_dieWithError($result);	
	}
}

sub _dieWithError{
	my $errorResult=shift;
	if ($errorResult->faultcode eq 'q0:FailedAuthentication'){
		die "Failed to login to remote server. The Sign-In E-Mail Address and/or password are incorrect.\n";
	}else{
		die join "\n", "Fault:",
			$errorResult->faultcode, 
			$errorResult->faultstring, 
			$errorResult->faultdetail;
	}	
}

sub _SOAPAuthArgs{
	my $url=shift;
	return SOAP::Data->prefix('e')->uri($WS_NS)->type('')
			->name("AuthenticationGUID")->value($authToken),			
		SOAP::Data->prefix('e')->uri($WS_NS)->type('')
			->name("ClientValidationGUID")->value($clientId),			
		#Commence pointless headers	
		SOAP::Header->uri($WSA_NS)->prefix("wsa")->name("Action" => $WS_NS),
		SOAP::Header->uri($WSA_NS)->prefix("wsa")->name("MessageID" => "urn:uuid:$$\@perl-client"),
		SOAP::Header->attr({"xmlns" => $WSA_NS})
			->name("ReplyTo" => [ SOAP::Header->name("Address" =>  'http://schemas.xmlsoap.org/ws/2004/08/addressing/role/anonymous') ]),
		SOAP::Header->uri($WSA_NS)->prefix("wsa")
			->name("To" => $url),
		SOAP::Header->type('')
			->attr({"xmlns" => "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd"})
			->name("Security" => [
				SOAP::Header->type('')->name("UsernameToken" => [
						SOAP::Header->type('')->name("Username" => $username),
						SOAP::Header->type('')->attr(
							{ "Type" => 'http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordText'}
						)->name("Password" => $password)
					]
				)
			]);
}

1;
__END__
